from collections import namedtuple

from django.core.exceptions import ImproperlyConfigured
from django.utils.functional import cached_property
from django.utils.safestring import mark_safe
from django.utils.text import slugify
from django.utils.translation import gettext_lazy as _

from wagtail.core import blocks as core_blocks
from wagtail.documents.blocks import DocumentChooserBlock
from wagtail.images.blocks import ImageChooserBlock

# MIXINS AND BASE CLASSES
# ------------------------------------------------------------------------------

Style = namedtuple('Style', ['name', 'label', 'css_class'])


class CSSClassMixin:
    """
    Add the CSS class defined for a block to its rendering context.
    """

    def get_css_classes(self, value):
        if hasattr(self.meta, 'css_class'):
            if isinstance(self.meta.css_class, str):
                return [self.meta.css_class]
            if not isinstance(self.meta.css_class, list):
                return list(self.meta.css_class)
            return self.meta.css_class
        return []

    def get_context(self, value, **kwargs):
        context = super().get_context(value, **kwargs)
        context['css_class'] = ' '.join(self.get_css_classes(value))
        return context


class StyleChoiceBlock(core_blocks.ChoiceBlock):
    class Meta:
        label = _("Style")

    def __init__(self, styles, default=None):
        self.styles = styles

        super().__init__(
            choices=[(s.name, s.label) for s in styles],
            default=default,
        )

    def to_python(self, value):
        return self.get_style_by_name(value)

    def get_prep_value(self, value):
        return value.name

    def value_from_form(self, value):
        return self.get_style_by_name(value)

    def value_for_form(self, value):
        return value and value.name

    def get_default(self):
        if isinstance(self.meta.default, Style):
            return self.meta.default
        return self.to_python(self.meta.default)

    def get_style_by_name(self, name):
        for style in self.styles:
            if style.name == name:
                return style


class StylizedStructBlock(CSSClassMixin, core_blocks.StructBlock):
    """
    A group of sub-blocks which defines an element with different styles.
    """

    styles = ()

    class Meta:
        default_style = None

    def __init__(self, *args, styles=None, **kwargs):
        super().__init__(*args, **kwargs)

        if styles is None:
            styles = self.styles

        if styles:
            style = StyleChoiceBlock(styles, default=self.meta.default_style)
            style.set_name('style')

            self.child_blocks['style'] = style

    def get_css_classes(self, value):
        css_classes = super().get_css_classes(value)
        style = value.get('style', None)
        if style and style.css_class:
            css_classes.append(style.css_class)
        return css_classes


# TYPOGRAPHY
# ------------------------------------------------------------------------------

HEADING_LEVELS = [
    (i, _("Level %(level)d heading") % {'level': i}) for i in range(2, 6)
]


class HeadingValue(core_blocks.StructValue):
    @cached_property
    def tag(self):
        """Return the HTML tag to use for the level."""
        return 'h{}'.format(self.get('level'))

    @cached_property
    def anchor(self):
        """Generate a slug from the title to be used as an anchor."""
        return slugify(self.get('text'))


class HeadingBlock(core_blocks.StructBlock):
    """
    A section heading with a choosable level.
    """

    text = core_blocks.CharBlock(label=_("Title"), classname='title')
    level = core_blocks.ChoiceBlock(
        choices=HEADING_LEVELS,
        default=2,
        label=_("Level"),
    )

    class Meta:
        icon = 'title'
        label = _("Title")
        template = 'wagtail_cblocks/heading_block.html'
        value_class = HeadingValue


class ParagraphBlock(core_blocks.RichTextBlock):
    """
    A paragraph with simple or customized features.
    """

    features = ('bold', 'italic', 'ol', 'ul', 'hr', 'link', 'document-link')

    def __init__(self, features=None, **kwargs):
        if features is None:
            features = self.features
        return super().__init__(features=features, **kwargs)

    class Meta:
        icon = 'pilcrow'
        label = _("Paragraph")
        template = 'wagtail_cblocks/paragraph_block.html'


# LINK AND BUTTONS
# ------------------------------------------------------------------------------


class LinkTargetBlock(core_blocks.StreamBlock):
    """
    The target of a link, used by `LinkBlock`.
    """

    page = core_blocks.PageChooserBlock(
        label=_("Page"), icon='doc-empty-inverse'
    )
    document = DocumentChooserBlock(label=_("Document"), icon='doc-full')
    image = ImageChooserBlock(label=_("Image"))
    url = core_blocks.URLBlock(label=_("External link"))
    anchor = core_blocks.CharBlock(
        label=_("Anchor link"),
        help_text=mark_safe(
            _(
                "An anchor in the current page, for example: "
                "<code>#target-id</code>."
            )
        ),
    )

    class Meta:
        icon = 'link'
        max_num = 1
        form_classname = 'link-target-block'


class LinkValue(core_blocks.StructValue):
    @cached_property
    def href(self):
        """Return the URL of the chosen target or `None` if it is undefined."""
        try:
            child_value = self['target'][0].value
        except (IndexError, KeyError):
            return None
        if hasattr(child_value, 'file') and hasattr(child_value.file, 'url'):
            href = child_value.file.url
        elif hasattr(child_value, 'url'):
            href = child_value.url
        else:
            href = child_value
        return href


class LinkBlock(core_blocks.StructBlock):
    """
    A link with a target chosen from a range of types - i.e. a page, an URL.
    """

    class Meta:
        icon = 'link'
        label = _("Link")
        value_class = LinkValue
        form_classname = 'link-block'

    def __init__(self, *args, required=True, **kwargs):
        super().__init__(*args, required=required, **kwargs)

        target = LinkTargetBlock(required=required)
        target.set_name('target')

        self.child_blocks['target'] = target

    @property
    def required(self):
        return self.meta.required


class ButtonBlock(StylizedStructBlock):
    """
    A button which acts like a link.
    """

    text = core_blocks.CharBlock(label=_("Text"))
    link = LinkBlock()

    class Meta:
        icon = 'link'
        label = _("Button")
        template = 'wagtail_cblocks/button_block.html'


# IMAGES
# ------------------------------------------------------------------------------


class ImageBlock(core_blocks.StructBlock):
    """
    An image with optional caption and link.
    """

    image = ImageChooserBlock(label=_("Image"))
    caption = core_blocks.CharBlock(required=False, label=_("Caption"))
    link = LinkBlock(required=False)

    class Meta:
        icon = 'image'
        label = _("Image")
        template = 'wagtail_cblocks/image_block.html'


# LAYOUT
# ------------------------------------------------------------------------------

HORIZONTAL_ALIGNMENTS = [
    ('start', _("Left")),
    ('center', _("Center")),
    ('end', _("Right")),
]
HORIZONTAL_ALIGNMENT_DEFAULT = None


class ColumnsBlock(core_blocks.StructBlock):
    """
    A list of columns which can be horizontally aligned.
    """

    horizontal_align = core_blocks.ChoiceBlock(
        choices=HORIZONTAL_ALIGNMENTS,
        default=HORIZONTAL_ALIGNMENT_DEFAULT,
        required=False,
        label=_("Horizontal alignment"),
    )

    class Meta:
        icon = 'table'
        label = _("Columns")
        template = 'wagtail_cblocks/columns_block.html'

    def __init__(self, column_block=None, **kwargs):
        super().__init__(**kwargs)

        if column_block is None:
            if not hasattr(self.meta, 'column_block'):
                raise ImproperlyConfigured(
                    "ColumnsBlock was not passed a 'column_block' object"
                )
            column_block = self.meta.column_block

        columns = core_blocks.ListBlock(
            column_block,
            collapsed=True,
            form_classname='columns-block-list',
            label=_("Columns"),
        )
        columns.set_name('columns')

        self.child_blocks['columns'] = columns
        self.child_blocks.move_to_end('columns', last=False)



from django.urls import reverse

from wagtail.tests.utils.form_data import (
    nested_form_data,
    rich_text,
    streamfield,
)

import pytest
from bs4 import BeautifulSoup
from pytest_django.asserts import assertContains


@pytest.mark.django_db
def test_insert_editor_css(admin_client, root_page):
    response = admin_client.get('/admin/pages/%d/edit/' % root_page.id)
    assertContains(response, 'wagtail_cblocks/css/editor.css')


@pytest.mark.django_db
class TestStylizedStructBlock:
    def test_page_preview(self, admin_client, root_page):
        add_url = reverse(
            'wagtailadmin_pages:add',
            args=('tests', 'standardpage', root_page.id),
        )

        form_data = nested_form_data(
            {
                'title': "A page",
                'body': streamfield(
                    [
                        (
                            'hero_block',
                            {
                                'style': 'centered',
                                'blocks': streamfield(
                                    [
                                        (
                                            'paragraph_block',
                                            rich_text('<p>Lorem ipsum</p>'),
                                        ),
                                    ]
                                ),
                            },
                        ),
                    ]
                ),
            }
        )
        response = admin_client.post(add_url, data=form_data)
        assert response.status_code == 200

        soup = BeautifulSoup(response.content, 'html5lib')
        preview_url = soup.select_one('.button.action-preview')['data-action']

        form_data['slug'] = 'a-page'
        response = admin_client.post(preview_url, data=form_data)
        assert response.status_code == 200
        assert response.json()['is_valid'] is True

        response = admin_client.get(preview_url)
        assert response.status_code == 200
        assertContains(response, 'hero-centered')
        assertContains(response, 'Lorem ipsum')



import os
from pathlib import Path

BASE_DIR = Path(__file__).parent

VAR_DIR = Path(__file__).parent / 'var'

DEBUG = True if os.environ.get('DEBUG', '0') == '1' else False

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': str(BASE_DIR / 'sqlite.db'),
    }
}

SECRET_KEY = 'not needed'

ALLOWED_HOSTS = ['localhost', 'testserver']

ROOT_URLCONF = 'tests.urls'

STATIC_URL = '/static/'

STATICFILES_FINDERS = [
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
]

STATIC_ROOT = VAR_DIR / 'static'

MEDIA_URL = '/media/'

MEDIA_ROOT = VAR_DIR / 'media'

USE_TZ = True

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'debug': DEBUG,
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

MIDDLEWARE = [
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

INSTALLED_APPS = [
    # django
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    # wagtail
    'wagtail.sites',
    'wagtail.users',
    'wagtail.documents',
    'wagtail.images',
    'wagtail.search',
    'wagtail.admin',
    'wagtail.core',
    'taggit',
    # wagtail_cblocks
    'wagtail_cblocks',
    'tests',
]

PASSWORD_HASHERS = ['django.contrib.auth.hashers.MD5PasswordHasher']

DEFAULT_AUTO_FIELD = 'django.db.models.AutoField'

# Wagtail settings

BASE_URL = 'http://testserver'

WAGTAIL_SITE_NAME = 'wagtail-cblocks test'

WAGTAILSEARCH_BACKENDS = {
    'default': {
        'BACKEND': 'wagtail.search.backends.database',
    }
}



# Generated by Django 4.0.3 on 2022-03-31 09:48

from django.db import migrations, models
import django.db.models.deletion
import wagtail.core.blocks
import wagtail.core.fields
import wagtail.documents.blocks
import wagtail.images.blocks
import wagtail_cblocks.blocks


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailcore', '0066_collection_management_permissions'),
    ]

    operations = [
        migrations.CreateModel(
            name='StandardPage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.page')),
                ('body', wagtail.core.fields.StreamField([('title_block', wagtail.core.blocks.StructBlock([('text', wagtail.core.blocks.CharBlock(form_classname='title', label='Title')), ('level', wagtail.core.blocks.ChoiceBlock(choices=[(2, 'Level 2 heading'), (3, 'Level 3 heading'), (4, 'Level 4 heading'), (5, 'Level 5 heading')], label='Level'))])), ('paragraph_block', wagtail_cblocks.blocks.ParagraphBlock()), ('button_block', wagtail.core.blocks.StructBlock([('text', wagtail.core.blocks.CharBlock(label='Text')), ('link', wagtail.core.blocks.StructBlock([('target', wagtail.core.blocks.StreamBlock([('page', wagtail.core.blocks.PageChooserBlock(icon='doc-empty-inverse', label='Page')), ('document', wagtail.documents.blocks.DocumentChooserBlock(icon='doc-full', label='Document')), ('image', wagtail.images.blocks.ImageChooserBlock(label='Image')), ('url', wagtail.core.blocks.URLBlock(label='External link')), ('anchor', wagtail.core.blocks.CharBlock(help_text='An anchor in the current page, for example: <code>#target-id</code>.', label='Anchor link'))], required=True))], required=True)), ('style', wagtail.core.blocks.ChoiceBlock(choices=[('primary', 'Primary'), ('secondary', 'Secondary'), ('primary-lg', 'Large primary')]))], default_style='primary-lg')), ('image_block', wagtail.core.blocks.StructBlock([('image', wagtail.images.blocks.ImageChooserBlock(label='Image')), ('caption', wagtail.core.blocks.CharBlock(label='Caption', required=False)), ('link', wagtail.core.blocks.StructBlock([('target', wagtail.core.blocks.StreamBlock([('page', wagtail.core.blocks.PageChooserBlock(icon='doc-empty-inverse', label='Page')), ('document', wagtail.documents.blocks.DocumentChooserBlock(icon='doc-full', label='Document')), ('image', wagtail.images.blocks.ImageChooserBlock(label='Image')), ('url', wagtail.core.blocks.URLBlock(label='External link')), ('anchor', wagtail.core.blocks.CharBlock(help_text='An anchor in the current page, for example: <code>#target-id</code>.', label='Anchor link'))], required=False))], required=False))])), ('hero_block', wagtail.core.blocks.StructBlock([('blocks', wagtail.core.blocks.StreamBlock([('title_block', wagtail.core.blocks.StructBlock([('text', wagtail.core.blocks.CharBlock(form_classname='title', label='Title')), ('level', wagtail.core.blocks.ChoiceBlock(choices=[(2, 'Level 2 heading'), (3, 'Level 3 heading'), (4, 'Level 4 heading'), (5, 'Level 5 heading')], label='Level'))])), ('paragraph_block', wagtail_cblocks.blocks.ParagraphBlock()), ('button_block', wagtail.core.blocks.StructBlock([('text', wagtail.core.blocks.CharBlock(label='Text')), ('link', wagtail.core.blocks.StructBlock([('target', wagtail.core.blocks.StreamBlock([('page', wagtail.core.blocks.PageChooserBlock(icon='doc-empty-inverse', label='Page')), ('document', wagtail.documents.blocks.DocumentChooserBlock(icon='doc-full', label='Document')), ('image', wagtail.images.blocks.ImageChooserBlock(label='Image')), ('url', wagtail.core.blocks.URLBlock(label='External link')), ('anchor', wagtail.core.blocks.CharBlock(help_text='An anchor in the current page, for example: <code>#target-id</code>.', label='Anchor link'))], required=True))], required=True)), ('style', wagtail.core.blocks.ChoiceBlock(choices=[('primary', 'Primary'), ('secondary', 'Secondary'), ('primary-lg', 'Large primary')]))], default_style='primary-lg')), ('image_block', wagtail.core.blocks.StructBlock([('image', wagtail.images.blocks.ImageChooserBlock(label='Image')), ('caption', wagtail.core.blocks.CharBlock(label='Caption', required=False)), ('link', wagtail.core.blocks.StructBlock([('target', wagtail.core.blocks.StreamBlock([('page', wagtail.core.blocks.PageChooserBlock(icon='doc-empty-inverse', label='Page')), ('document', wagtail.documents.blocks.DocumentChooserBlock(icon='doc-full', label='Document')), ('image', wagtail.images.blocks.ImageChooserBlock(label='Image')), ('url', wagtail.core.blocks.URLBlock(label='External link')), ('anchor', wagtail.core.blocks.CharBlock(help_text='An anchor in the current page, for example: <code>#target-id</code>.', label='Anchor link'))], required=False))], required=False))])), ('columns_block', wagtail.core.blocks.StructBlock([('columns', wagtail.core.blocks.ListBlock(wagtail.core.blocks.StreamBlock([('title_block', wagtail.core.blocks.StructBlock([('text', wagtail.core.blocks.CharBlock(form_classname='title', label='Title')), ('level', wagtail.core.blocks.ChoiceBlock(choices=[(2, 'Level 2 heading'), (3, 'Level 3 heading'), (4, 'Level 4 heading'), (5, 'Level 5 heading')], label='Level'))])), ('paragraph_block', wagtail_cblocks.blocks.ParagraphBlock()), ('button_block', wagtail.core.blocks.StructBlock([('text', wagtail.core.blocks.CharBlock(label='Text')), ('link', wagtail.core.blocks.StructBlock([('target', wagtail.core.blocks.StreamBlock([('page', wagtail.core.blocks.PageChooserBlock(icon='doc-empty-inverse', label='Page')), ('document', wagtail.documents.blocks.DocumentChooserBlock(icon='doc-full', label='Document')), ('image', wagtail.images.blocks.ImageChooserBlock(label='Image')), ('url', wagtail.core.blocks.URLBlock(label='External link')), ('anchor', wagtail.core.blocks.CharBlock(help_text='An anchor in the current page, for example: <code>#target-id</code>.', label='Anchor link'))], required=True))], required=True)), ('style', wagtail.core.blocks.ChoiceBlock(choices=[('primary', 'Primary'), ('secondary', 'Secondary'), ('primary-lg', 'Large primary')]))], default_style='primary-lg')), ('image_block', wagtail.core.blocks.StructBlock([('image', wagtail.images.blocks.ImageChooserBlock(label='Image')), ('caption', wagtail.core.blocks.CharBlock(label='Caption', required=False)), ('link', wagtail.core.blocks.StructBlock([('target', wagtail.core.blocks.StreamBlock([('page', wagtail.core.blocks.PageChooserBlock(icon='doc-empty-inverse', label='Page')), ('document', wagtail.documents.blocks.DocumentChooserBlock(icon='doc-full', label='Document')), ('image', wagtail.images.blocks.ImageChooserBlock(label='Image')), ('url', wagtail.core.blocks.URLBlock(label='External link')), ('anchor', wagtail.core.blocks.CharBlock(help_text='An anchor in the current page, for example: <code>#target-id</code>.', label='Anchor link'))], required=False))], required=False))]))]), collapsed=True, form_classname='columns-block-list', label='Columns')), ('horizontal_align', wagtail.core.blocks.ChoiceBlock(choices=[('start', 'Left'), ('center', 'Center'), ('end', 'Right')], label='Horizontal alignment', required=False))]))], label='Content')), ('style', wagtail.core.blocks.ChoiceBlock(choices=[('centered', 'Centered'), ('responsive', 'Responsive'), ('dark', 'Dark')]))])), ('columns_block', wagtail.core.blocks.StructBlock([('columns', wagtail.core.blocks.ListBlock(wagtail.core.blocks.StreamBlock([('title_block', wagtail.core.blocks.StructBlock([('text', wagtail.core.blocks.CharBlock(form_classname='title', label='Title')), ('level', wagtail.core.blocks.ChoiceBlock(choices=[(2, 'Level 2 heading'), (3, 'Level 3 heading'), (4, 'Level 4 heading'), (5, 'Level 5 heading')], label='Level'))])), ('paragraph_block', wagtail_cblocks.blocks.ParagraphBlock()), ('button_block', wagtail.core.blocks.StructBlock([('text', wagtail.core.blocks.CharBlock(label='Text')), ('link', wagtail.core.blocks.StructBlock([('target', wagtail.core.blocks.StreamBlock([('page', wagtail.core.blocks.PageChooserBlock(icon='doc-empty-inverse', label='Page')), ('document', wagtail.documents.blocks.DocumentChooserBlock(icon='doc-full', label='Document')), ('image', wagtail.images.blocks.ImageChooserBlock(label='Image')), ('url', wagtail.core.blocks.URLBlock(label='External link')), ('anchor', wagtail.core.blocks.CharBlock(help_text='An anchor in the current page, for example: <code>#target-id</code>.', label='Anchor link'))], required=True))], required=True)), ('style', wagtail.core.blocks.ChoiceBlock(choices=[('primary', 'Primary'), ('secondary', 'Secondary'), ('primary-lg', 'Large primary')]))], default_style='primary-lg')), ('image_block', wagtail.core.blocks.StructBlock([('image', wagtail.images.blocks.ImageChooserBlock(label='Image')), ('caption', wagtail.core.blocks.CharBlock(label='Caption', required=False)), ('link', wagtail.core.blocks.StructBlock([('target', wagtail.core.blocks.StreamBlock([('page', wagtail.core.blocks.PageChooserBlock(icon='doc-empty-inverse', label='Page')), ('document', wagtail.documents.blocks.DocumentChooserBlock(icon='doc-full', label='Document')), ('image', wagtail.images.blocks.ImageChooserBlock(label='Image')), ('url', wagtail.core.blocks.URLBlock(label='External link')), ('anchor', wagtail.core.blocks.CharBlock(help_text='An anchor in the current page, for example: <code>#target-id</code>.', label='Anchor link'))], required=False))], required=False))]))]), collapsed=True, form_classname='columns-block-list', label='Columns')), ('horizontal_align', wagtail.core.blocks.ChoiceBlock(choices=[('start', 'Left'), ('center', 'Center'), ('end', 'Right')], label='Horizontal alignment', required=False))])), ('row_columns_block', wagtail.core.blocks.StructBlock([('columns', wagtail.core.blocks.ListBlock(wagtail.core.blocks.StreamBlock([('title_block', wagtail.core.blocks.StructBlock([('text', wagtail.core.blocks.CharBlock(form_classname='title', label='Title')), ('level', wagtail.core.blocks.ChoiceBlock(choices=[(2, 'Level 2 heading'), (3, 'Level 3 heading'), (4, 'Level 4 heading'), (5, 'Level 5 heading')], label='Level'))])), ('paragraph_block', wagtail_cblocks.blocks.ParagraphBlock()), ('button_block', wagtail.core.blocks.StructBlock([('text', wagtail.core.blocks.CharBlock(label='Text')), ('link', wagtail.core.blocks.StructBlock([('target', wagtail.core.blocks.StreamBlock([('page', wagtail.core.blocks.PageChooserBlock(icon='doc-empty-inverse', label='Page')), ('document', wagtail.documents.blocks.DocumentChooserBlock(icon='doc-full', label='Document')), ('image', wagtail.images.blocks.ImageChooserBlock(label='Image')), ('url', wagtail.core.blocks.URLBlock(label='External link')), ('anchor', wagtail.core.blocks.CharBlock(help_text='An anchor in the current page, for example: <code>#target-id</code>.', label='Anchor link'))], required=True))], required=True)), ('style', wagtail.core.blocks.ChoiceBlock(choices=[('primary', 'Primary'), ('secondary', 'Secondary'), ('primary-lg', 'Large primary')]))], default_style='primary-lg')), ('image_block', wagtail.core.blocks.StructBlock([('image', wagtail.images.blocks.ImageChooserBlock(label='Image')), ('caption', wagtail.core.blocks.CharBlock(label='Caption', required=False)), ('link', wagtail.core.blocks.StructBlock([('target', wagtail.core.blocks.StreamBlock([('page', wagtail.core.blocks.PageChooserBlock(icon='doc-empty-inverse', label='Page')), ('document', wagtail.documents.blocks.DocumentChooserBlock(icon='doc-full', label='Document')), ('image', wagtail.images.blocks.ImageChooserBlock(label='Image')), ('url', wagtail.core.blocks.URLBlock(label='External link')), ('anchor', wagtail.core.blocks.CharBlock(help_text='An anchor in the current page, for example: <code>#target-id</code>.', label='Anchor link'))], required=False))], required=False))]))]), collapsed=True, form_classname='columns-block-list', label='Columns')), ('horizontal_align', wagtail.core.blocks.ChoiceBlock(choices=[('start', 'Left'), ('center', 'Center'), ('end', 'Right')], label='Horizontal alignment', required=False)), ('layout', wagtail.core.blocks.ChoiceBlock(choices=[('2', '2-columns'), ('3', '3-columns'), ('4', '4-columns'), ('auto', 'natural width')]))]))])),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page',),
        ),
    ]



#!/usr/bin/env python
import os
import sys
from pathlib import Path

sys.path.insert(0, str(Path(__file__).parent.parent.resolve()))

if __name__ == '__main__':
    os.environ.setdefault('DEBUG', '1')
    os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'tests.settings')

    from django.core.management import execute_from_command_line

    execute_from_command_line(sys.argv)



import sys

import nox

nox.options.sessions = ['tests_wagtail215', 'tests_wagtail30']

dj32 = nox.param('3.2', id='dj32')
dj40 = nox.param('4.0', id='dj40')


def install_and_run_tests(session):
    session.install('-r', 'requirements-test.txt')
    session.install('-e', '.[factories]')
    tests = session.posargs or ['tests/']
    session.run(
        'pytest',
        '--cov',
        '--cov-config=pyproject.toml',
        '--cov-report=',
        *tests,
        env={'COVERAGE_FILE': f'.coverage.{session.name}'},
    )
    session.notify('coverage')


@nox.session
@nox.parametrize('django', [dj32])
def tests_wagtail215(session, django):
    session.install(f'django=={django}')
    session.install('wagtail==2.15')
    install_and_run_tests(session)


@nox.session
@nox.parametrize('django', [dj32, dj40])
def tests_wagtail30(session, django):
    if django == '4.0' and sys.version_info.minor < 8:
        session.skip("Django 4.0 requires Python 3.8+")
    session.install(f'django=={django}')
    session.install('wagtail==3.0')
    install_and_run_tests(session)


@nox.session
def coverage(session):
    session.install('coverage[toml]')
    session.run('coverage', 'combine')
    session.run('coverage', 'report', '--show-missing')
    session.run('coverage', 'xml')
    session.run('coverage', 'erase')


@nox.session
def lint(session):
    session.install('pre-commit')
    session.run(
        'pre-commit',
        'run',
        '--all-files',
        '--show-diff-on-failure',
        '--hook-stage=manual',
        *session.posargs,
    )
